import datetime
import traceback
import uuid

from cpl_core.time import TimeFormatSettings
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord.ext import commands
from discord.ext.commands import Context, CommandError

from cpl_core.logging import LoggerABC
from cpl_discord.events.on_command_error_abc import OnCommandErrorABC

from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.configuration.bot_settings import BotSettings
from bot_core.exception.check_error import CheckError


class BaseOnCommandErrorEvent(OnCommandErrorABC):
    def __init__(
        self,
        logger: LoggerABC,
        bot: DiscordBotServiceABC,
        messenger: MessageServiceABC,
        bot_settings: BotSettings,
        time_format_settings: TimeFormatSettings,
        translate: TranslatePipe,
    ):
        OnCommandErrorABC.__init__(self)
        self._logger = logger
        self._bot = bot
        self._messenger = messenger
        self._bot_settings = bot_settings
        self._time_format_settings = time_format_settings
        self._t = translate

    async def on_command_error(self, ctx: Context, error: CommandError):
        if isinstance(error, CheckError):
            return

        error = getattr(error, "original", error)
        uid = uuid.uuid4()
        self._logger.error(__name__, f"Got error: {type(error).__name__} UID: {uid}")

        if isinstance(error, commands.MissingRequiredArgument):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.missing_required_argument"),
                without_tracking=True,
            )

        elif isinstance(error, commands.ArgumentParsingError):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.argument_parsing_error"),
                without_tracking=True,
            )

        elif isinstance(error, commands.UnexpectedQuoteError):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.unexpected_quote_error"),
                without_tracking=True,
            )

        elif isinstance(error, commands.InvalidEndOfQuotedStringError):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.invalid_end_of_quoted_string_error"),
                without_tracking=True,
            )

        elif isinstance(error, commands.ExpectedClosingQuoteError):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.expected_closing_quote_error"),
                without_tracking=True,
            )

        elif isinstance(error, commands.BadArgument):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.bad_argument"),
                without_tracking=True,
            )

        elif isinstance(error, commands.BadUnionArgument):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.bad_union_argument"),
                without_tracking=True,
            )

        elif isinstance(error, commands.PrivateMessageOnly):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.private_message_only"),
                without_tracking=True,
            )

        elif isinstance(error, commands.NoPrivateMessage):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.no_private_message"),
                without_tracking=True,
            )

        elif isinstance(error, commands.CheckFailure):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.check_failure"),
                without_tracking=True,
            )

        elif isinstance(error, commands.CheckAnyFailure):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.check_any_failure"),
                without_tracking=True,
            )

        elif isinstance(error, commands.CommandNotFound):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.command_not_found"),
                without_tracking=True,
            )

        elif isinstance(error, commands.DisabledCommand):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.disabled_command"),
                without_tracking=True,
            )

        elif isinstance(error, commands.CommandInvokeError):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.command_invoke_error"),
                without_tracking=True,
            )

        elif isinstance(error, commands.TooManyArguments):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.too_many_arguments"),
                without_tracking=True,
            )

        elif isinstance(error, commands.UserInputError):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.user_input_error"),
                without_tracking=True,
            )

        elif isinstance(error, commands.CommandOnCooldown):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.command_on_cooldown"),
                without_tracking=True,
            )

        elif isinstance(error, commands.MaxConcurrencyReached):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.max_concurrency_reached"),
                without_tracking=True,
            )

        elif isinstance(error, commands.NotOwner):
            await self._messenger.send_ctx_msg(ctx, self._t.transform("common.errors.not_owner"), without_tracking=True)

        elif isinstance(error, commands.MissingPermissions):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.missing_permissions"),
                without_tracking=True,
            )

        elif isinstance(error, commands.BotMissingPermissions):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.bot_missing_permissions"),
                without_tracking=True,
            )

        elif isinstance(error, commands.MissingRole):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.missing_role"),
                without_tracking=True,
            )

        elif isinstance(error, commands.BotMissingRole):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.bot_missing_role"),
                without_tracking=True,
            )

        elif isinstance(error, commands.MissingAnyRole):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.missing_any_role"),
                without_tracking=True,
            )

        elif isinstance(error, commands.BotMissingAnyRole):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.bot_missing_any_role"),
                without_tracking=True,
            )

        elif isinstance(error, commands.NSFWChannelRequired):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.nsfw_channel_required"),
                without_tracking=True,
            )

        elif isinstance(error, commands.ExtensionError):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.extension_error"),
                without_tracking=True,
            )

        elif isinstance(error, commands.ExtensionAlreadyLoaded):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.extension_already_loaded"),
                without_tracking=True,
            )

        elif isinstance(error, commands.ExtensionNotLoaded):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.extension_not_loaded"),
                without_tracking=True,
            )

        elif isinstance(error, commands.NoEntryPointError):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.no_entry_point_error"),
                without_tracking=True,
            )

        elif isinstance(error, commands.ExtensionFailed):
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.extension_failed"),
                without_tracking=True,
            )

        else:
            await self._messenger.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.command_error"),
                without_tracking=True,
            )
            message = self._t.transform("modules.base.technician_command_error_message").format(
                ctx.command,
                ctx.author,
                error,
                datetime.datetime.now().strftime(self._time_format_settings.date_time_format),
                uid,
            )
            for t in self._bot_settings.technicians:
                member = self._bot.get_user(t)
                await self._messenger.send_dm_message(message, member, without_tracking=True)
